zebkit.package("ui", function(pkg, Class) {
    /**
     * Class that holds mouse cursor constant.
     * @constructor
     * @class zebkit.ui.Cursor
     */
    pkg.Cursor = {
        /**
         * "default"
         * @const DEFAULT
         * @type {String}
         */
        DEFAULT: "default",

        /**
         * "move"
         * @const MOVE
         * @type {String}
         */
        MOVE: "move",

        /**
         * "wait"
         * @const WAIT
         * @type {String}
         */
        WAIT: "wait",

        /**
         * "text"
         * @const TEXT
         * @type {String}
         */
        TEXT: "text",

        /**
         * "pointer"
         * @const HAND
         * @type {String}
         */
        HAND: "pointer",

        /**
         * "ne-resize"
         * @const NE_RESIZE
         * @type {String}
         */
        NE_RESIZE: "ne-resize",

        /**
         * "sw-resize"
         * @const SW_RESIZE
         * @type {String}
         */
        SW_RESIZE: "sw-resize",

        /**
         * "se-resize"
         * @const SE_RESIZE
         * @type {String}
         */
        SE_RESIZE: "se-resize",

        /**
         * "nw-resize"
         * @const NW_RESIZE
         * @type {String}
         */
        NW_RESIZE: "nw-resize",

        /**
         * "s-resize"
         * @const S_RESIZE
         * @type {String}
         */
        S_RESIZE: "s-resize",

        /**
         * "w-resize"
         * @const W_RESIZE
         * @type {String}
         */
        W_RESIZE: "w-resize",

        /**
         * "n-resize"
         * @const N_RESIZE
         * @type {String}
         */
        N_RESIZE: "n-resize",

        /**
         * "e-resize"
         * @const E_RESIZE
         * @type {String}
         */
        E_RESIZE: "e-resize",

        /**
         * "col-resize"
         * @const COL_RESIZE
         * @type {String}
         */
        COL_RESIZE: "col-resize",

        /**
         * "help"
         * @const HELP
         * @type {String}
         */
        HELP: "help"
    };

    /**
     *  UI component render class. Renders the given target UI component
     *  on the given surface using the specified 2D context
     *  @param {zebkit.layout.Layoutable} [target] an UI component to be rendered
     *  @class zebkit.ui.CompRender
     *  @constructor
     *  @extends zebkit.draw.Render
     */
    pkg.CompRender = Class(zebkit.draw.Render, [
        function $prototype() {
            /**
             * Get preferred size of the render. The method doesn't calculates
             * preferred size it simply calls the target component "getPreferredSize"
             * method.
             * @method getPreferredSize
             * @return {Object} a preferred size
             *
             *      {width:<Integer>, height: <Integer>}
             */
            this.getPreferredSize = function(){
                return this.target === null || this.target.isVisible === false ? { width:0, height:0 }
                                                                               : this.target.getPreferredSize();
            };

            this.paint = function(g,x,y,w,h,d){
                var c = this.target;
                if (c !== null && c.isVisible) {
                    var prevW  = -1,
                        prevH  = 0,
                        parent = null;

                    if (w !== c.width || h !== c.height) {
                        if (c.getCanvas() !== null) {
                            parent = c.parent;
                            c.parent = null;
                        }

                        prevW = c.width;
                        prevH = c.height;
                        c.setSize(w, h);
                    }

                    // validate should be done here since setSize can be called
                    // above
                    c.validate();
                    g.translate(x, y);

                    try {
                        c.paintComponent(g);
                    } catch(e) {
                        if (parent !== null) {
                            c.parent = parent;
                        }
                        g.translate(-x, -y);
                        throw e;
                    }
                    g.translate(-x, -y);

                    if (prevW >= 0){
                        c.setSize(prevW, prevH);
                        if (parent !== null) {
                            c.parent = parent;
                        }
                        c.validate();
                    }
                }
            };
        }
    ]);

    /**
     * Shortcut to create a UI component by the given description. Depending on the description type
     * the following components are created:
     *
     *    - **String**
     *       - String encoded as "[x] Text" or "[] Text" will considered as checkbox component
     *       - String encoded as "@(image_path:WxH) Text" will considered as image or image label component
     *       - All other strings will be considered as label component
     *    - **Array** zebkit.ui.Combobox
     *    - **2D Array** zebkit.ui.grid.Grid
     *    - **Image** will be embedded with zebkit.ui.ImagePan component
     *    - **zebkit.ui.View instance** will be embedded with zebkit.ui.ViewPan component
     *    - **zebkit.ui.Panel instance** will be returned as is
     *
     * @method $component
     * @protected
     * @for  zebkit.ui
     * @param  {Object} desc a description
     * @return {zebkit.ui.Panel}  a created UI component
     */
    pkg.$component = function(desc, instance) {
        var hasInstance = arguments.length > 1;

        if (zebkit.isString(desc)) {
            //  [x] Text
            //  @(image-path:WxH) label
            //  %{<json> json-path}  !!! not supported
            //  { "zebkit.ui.Panel" }

            var m   = desc.match(/^(\[[x ]?\])/),
                txt = null;

            if (m !== null) {
                txt = desc.substring(m[1].length);
                var ch  = hasInstance && instance.clazz.Checkbox !== undefined ? new instance.clazz.Checkbox(txt)
                                                                               : new pkg.Checkbox(txt);
                ch.setValue(m[1].indexOf('x') > 0);
                return ch;
            } else {
                m = desc.match(/^@\((.*)\)(\:[0-9]+x[0-9]+)?/);
                if (m !== null) {
                    var path = m[1];

                    txt  = desc.substring(path.length + 3 + (m[2] !== undefined ? m[2].length : 0)).trim();

                    var img = hasInstance && instance.clazz.ImagePan !== undefined ? new instance.clazz.ImagePan(path)
                                                                                   : new pkg.ImagePan(path);

                    if (m[2] !== undefined) {
                        var s = m[2].substring(1).split('x'),
                            w = parseInt(s[0], 10),
                            h = parseInt(s[1], 10);

                        img.setPreferredSize(w, h);
                    }

                    if (txt.length === 0) {
                        return img;
                    } else {
                        return hasInstance && instance.clazz.ImageLabel !== undefined ? new instance.clazz.ImageLabel(txt, img)
                                                                                      : new pkg.ImageLabel(txt, img);
                    }
                } else {
                    return hasInstance && instance.clazz.Label !== undefined ? new instance.clazz.Label(desc)
                                                                             : new pkg.Label(desc);
                }
            }
        } else if (Array.isArray(desc)) {
            if (desc.length > 0 && Array.isArray(desc[0])) {
                var model = new zebkit.data.Matrix(desc.length, desc[0].length);
                for(var row = 0; row < model.rows; row++) {
                    for(var col = 0; col < model.cols; col++) {
                        model.put(row, col, desc[row][col]);
                    }
                }
                return new pkg.grid.Grid(model);
            } else {
                var clz = hasInstance && instance.clazz.Combo !== undefined ? instance.clazz.Combo
                                                                            : pkg.Combo,
                    combo = new clz(new clz.CompList(true)),
                    selectedIndex = -1;

                for(var i = 0; i < desc.length; i++) {
                    var ss = desc[i];
                    if (zebkit.isString(ss)) {
                        if (selectedIndex === -1 && ss.length > 1 && ss[0] === '*') {
                            selectedIndex = i;
                            desc[i] = ss.substring(1);
                        }
                    }
                    combo.list.add(pkg.$component(desc[i], combo.list));
                }

                combo.select(selectedIndex);
                return combo;
            }
        } else if (desc instanceof Image) {
            return hasInstance && instance.clazz.ImagePan !== undefined ? new instance.clazz.ImagePan(desc)
                                                                        : new pkg.ImagePan(desc);
        } else if (zebkit.instanceOf(desc, zebkit.draw.View)) {
            var v = hasInstance && instance.clazz.ViewPan !== undefined ? new instance.clazz.ViewPan()
                                                                        : new pkg.ViewPan();
            v.setView(desc);
            return v;
        } else if (zebkit.instanceOf(desc, pkg.Panel)) {
            return desc;
        } else {
            throw new Error("Invalid component description '" +  desc + "'");
        }
    };

    /**
     * Named views holder interface.
     * @class  zebkit.ui.HostDecorativeViews
     * @interface  zebkit.ui.HostDecorativeViews
     */
    pkg.HostDecorativeViews = zebkit.Interface([
        function $prototype() {
            /**
             * Set views set.
             * @param {Object} v named views set.
             * @method setViews
             * @chainable
             */
            this.setViews = function(v){
                if (this.views === undefined) {
                    this.views = {};
                }

                var b = false;
                for(var k in v) {
                    if (v.hasOwnProperty(k)) {
                        var nv = zebkit.draw.$view(v[k]);
                        if (this.views[k] !== nv) {
                            this.views[k] = nv;
                            b = true;
                        }
                    }
                }

                if (b === true) {
                    this.vrp();
                }

                return this;
            };
        }
    ]);

    /**
     *  UI component to keep and render the given "zebkit.draw.View" class
     *  instance. The target view defines the component preferred size
     *  and the component view.
     *  @class zebkit.ui.ViewPan
     *  @constructor
     *  @extends zebkit.ui.Panel
     */
    pkg.ViewPan = Class(pkg.Panel, [
        function $prototype() {
            /**
             * Reference to a view that the component visualize
             * @attribute view
             * @type {zebkit.draw.View}
             * @default null
             * @readOnly
             */
            this.view = null;

            this.paint = function (g){
                if (this.view !== null){
                    var l = this.getLeft(),
                        t = this.getTop();

                    this.view.paint(g, l, t, this.width  - l - this.getRight(),
                                             this.height - t - this.getBottom(), this);
                }
            };

            /**
             * Set the target view to be wrapped with the UI component
             * @param  {zebkit.draw.View|Function} v a view or a rendering
             * view "paint(g,x,y,w,h,c)" function
             * @method setView
             * @chainable
             */
            this.setView = function(v) {
                var old = this.view;
                v = zebkit.draw.$view(v);

                if (v !== old) {
                    this.view = v;
                    this.notifyRender(old, v);
                    this.vrp();
                }

                return this;
            };

            /**
             * Override the parent method to calculate preferred size basing on a target view.
             * @param  {zebkit.ui.Panel} t a target container
             * @return {Object} return a target view preferred size if it is defined.
             * The returned structure is the following:
             *
             *     { width: {Integer}, height:{Integer} }
             *
             *  @method  calcPreferredSize
             */
            this.calcPreferredSize = function(t) {
                return this.view !== null ? this.view.getPreferredSize() : { width:0, height:0 };
            };
        }
    ]);

    /**
     *  Image panel UI component class. The component renders an image.
     *  @param {String|Image} [img] a path or direct reference to an image object.
     *  If the passed parameter is string it considered as path to an image.
     *  In this case the image will be loaded using the passed path.
     *  @param {Integer} [w] a preferred with of the image
     *  @param {Integer} [h] a preferred height of the image
     *  @class zebkit.ui.ImagePan
     *  @constructor
     *  @extends zebkit.ui.ViewPan
     */
    pkg.ImagePan = Class(pkg.ViewPan, [
        function(img, w, h) {
            this.setImage(arguments.length > 0 ? img : null);
            this.$super();
            if (arguments.length > 1) {
                this.setPreferredSize(w, arguments < 3 ? w : h);
            }
        },

        function $prototype() {
            /**
             * Image loader runner.
             * @attribute $runner
             * @readOnly
             * @private
             * @type {zebkit.DoIt}
             */
            this.$runner = null;

            /**
             * Set image to be rendered in the UI component
             * @method setImage
             * @param {String|Image|zebkit.draw.Picture} img a path or direct reference to an
             * image or zebkit.draw.Picture render.
             * If the passed parameter is string it considered as path to an image.
             * In this case the image will be loaded using the passed path
             * @chainable
             */
            this.setImage = function(img) {
                var $this = this;

                if (img !== null) {
                    var isPic = zebkit.instanceOf(img, zebkit.draw.Picture);
                    this.setView(isPic ? img : null);

                    this.$runner = zebkit.image(isPic ? img.target : img);
                    this.$runner.then(function(img) {
                        $this.$runner = null;

                        if (isPic === false) {
                            $this.setView(new zebkit.draw.Picture(img));
                        }

                        $this.vrp();

                        if ($this.imageLoaded !== undefined) {
                            $this.imageLoaded(img);
                        }

                        // fire imageLoaded event to children
                        for(var t = $this.parent; t !== null; t = t.parent){
                            if (t.childImageLoaded !== undefined) {
                                t.childImageLoaded(img);
                            }
                        }
                    }).catch(function(e) {
                        zebkit.dumpError(e);
                        $this.$runner = null;
                        $this.setView(null);
                    });
                } else if (this.$runner === null) {
                    this.setView(null);
                } else {
                    this.$runner.then(function() {
                        $this.setView(null);
                    });
                }
                return this;
            };
        }
    ]);

    /**
     * Line UI component class. Draw series of vertical or horizontal lines of using
     * the given line width and color. Vertical or horizontal line rendering s selected
     * depending on the line component size: if height is greater than width than vertical
     * line will be rendered.
     * @constructor
     * @param {String} [colors]* line colors
     * @class zebkit.ui.Line
     * @extends zebkit.ui.Panel
     */
    pkg.Line = Class(pkg.Panel, [
        function() {
            /**
             * Line colors
             * @attribute colors
             * @type {Array}
             * @readOnly
             * @default [ "gray" ]
             */
            this.$super();

            if (arguments.length > 0) {
                this.setColors.apply(this, arguments);
            }
        },

        function $prototype() {
            /**
             * Line colors set.
             * @attribute colors
             * @type {Array}
             * @readOnly
             * @default [ "gray" ]
             */
            this.colors = [ "gray" ];

            /**
             * Line width
             * @attribute lineWidth
             * @type {Integer}
             * @default 1
             */
            this.lineWidth = 1;

            /**
             * Line direction attribute. Can be "vertical" or "horizontal" or null value.
             * @attribute direction
             * @type {String}
             * @default null
             */
            this.direction = null;

            /**
             * Set line color.
             * @param {String} c a color
             * @method  setColor
             * @chainable
             * @readOnly
             */
            this.setColor = function(c) {
                this.setColors(c);
                return this;
            };

            /**
             * Set set of colors to be used to paint the line. Number of colors defines the number of
             * lines to be painted.
             * @param {String} colors* colors
             * @method setLineColors
             * @chainable
             */
            this.setColors = function() {
                this.colors = (arguments.length === 1) ? (Array.isArray(arguments[0]) ? arguments[0].slice(0)
                                                                                      : [ arguments[0] ] )
                                                       : Array.prototype.slice.call(arguments);
                this.repaint();
                return this;
            };

            /**
             * Set the given line direction.
             * @param {String} d a line direction. Can be "vertical" or "horizontal" or null value.
             * null means auto detected direction.
             * @method setDirection
             */
            this.setDirection = function(d) {
                if (d !== this.direction) {
                    this.direction = d;
                    this.vrp();
                }
                return this;
            };

            this.paint = function(g) {
                var isHor  = this.direction === null ? this.width > this.height
                                                     : this.direction === "horizontal",
                    left   = this.getLeft(),
                    right  = this.getRight(),
                    top    = this.getTop(),
                    bottom = this.getBottom(),
                    xy     = isHor ? top : left;

                for(var i = 0; i < this.colors.length; i++) {
                    if (this.colors[i] !== null) {
                        g.setColor(this.colors[i]);
                        if (isHor === true) {
                            g.drawLine(this.left, xy, this.width - right - left, xy, this.lineWidth);
                        } else {
                            g.drawLine(xy, top, xy, this.height - top - bottom, this.lineWidth);
                        }
                    }
                    xy += this.lineWidth;
                }
            };

            this.calcPreferredSize = function(target) {
                var s = this.colors.length * this.lineWidth;
                return { width: s, height:s};
            };
        }
    ]);

    /**
     * Label UI component class. The label can be used to visualize simple string or multi lines text or
     * the given text render implementation:
     *
     *       // render simple string
     *       var l = new zebkit.ui.Label("Simple string");
     *
     *       // render multi lines text
     *       var l = new zebkit.ui.Label(new zebkit.data.Text("Multiline\ntext"));
     *
     *       // render password text
     *       var l = new zebkit.ui.Label(new zebkit.draw.PasswordText("password"));
     *
     * @param  {String|zebkit.data.TextModel|zebkit.draw.TextRender} [r] a text to be shown with the label.
     * You can pass a simple string or an instance of a text model or an instance of text render as the
     * text value.
     * @class zebkit.ui.Label
     * @constructor
     * @extends zebkit.ui.ViewPan
     */
    pkg.Label = Class(pkg.ViewPan, [
        function (r) {
            if (arguments.length === 0) {
                this.setView(new zebkit.draw.StringRender(""));
            } else {
                // test if input string is string
                if (typeof r === "string" || r.constructor === String) {
                    this.setView(r.length === 0 || r.indexOf('\n') >= 0 ? new zebkit.draw.TextRender(new zebkit.data.Text(r))
                                                                        : new zebkit.draw.StringRender(r));
                } else if (r.clazz         !== undefined &&
                           r.getTextLength !== undefined &&   // a bit faster than instanceOf checking if
                           r.getLines      !== undefined   )  // test if this is an instance of zebkit.data.TextModel
                {
                    this.setView(new zebkit.draw.TextRender(r));
                } else {
                    this.setView(r);
                }
            }
            this.$super();
        },

        function $prototype() {
            /**
             * Get the label text
             * @return {String} a zebkit label text
             * @method getValue
             */
            this.getValue = function() {
                return this.view.toString();
            };

            /**
             * Set the text field text model
             * @param  {zebkit.data.TextModel|String} m a text model to be set
             * @method setModel
             * @chainable
             */
            this.setModel = function(m) {
                this.setView(zebkit.isString(m) ? new zebkit.draw.StringRender(m)
                                                : new zebkit.draw.TextRender(m));
                return this;
            };

            /**
             * Get a text model
             * @return {zebkit.data.TextModel} a text model
             * @method getModel
             */
            this.getModel = function() {
                return this.view !== null ? this.view.target : null;
            };

            /**
             * Get the label text color
             * @return {String} a zebkit label color
             * @method getColor
             */
            this.getColor = function (){
                return this.view.color;
            };

            /**
             * Get the label text font
             * @return {zebkit.Font} a zebkit label font
             * @method getFont
             */
            this.getFont = function (){
                return this.view.font;
            };

            /**
             * Set the label text value
             * @param  {String} s a new label text
             * @method setValue
             * @chainable
             */
            this.setValue = function(s){
                if (s === null) {
                    s = "";
                }

                var old = this.view.toString();
                if (old !== s) {
                    this.view.setValue(s);
                    this.repaint();
                }

                return this;
            };

            /**
             * Set the label text color
             * @param  {String} c a text color
             * @method setColor
             * @chainable
             */
            this.setColor = function(c) {
                var old = this.view.color;
                if (old !== c) {
                    this.view.setColor(c);
                    this.repaint();
                }
                return this;
            };

            /**
             * Set the label text font
             * @param  {zebkit.Font} f a text font
             * @method setFont
             * @chainable
             */
            this.setFont = function(f) {
                var old = this.view.font;
                this.view.setFont.apply(this.view, arguments);
                if (old != this.view.font) {
                    this.repaint();
                }
                return this;
            };
        }
    ]);


    /**
     * Shortcut class to render bold text in Label
     * @param {String|zebkit.draw.TextRender|zebkit.data.TextModel} [t] a text string,
     * text model or text render instance
     * @constructor
     * @class zebkit.ui.BoldLabel
     * @extends zebkit.ui.Label
     */
    pkg.BoldLabel = Class(pkg.Label, []);


    /**
     * Image label UI component. This is UI container that consists from an image
     * component and an label component.Image is located at the left size of text.
     * @param {Image|String} [img] an image or path to the image
     * @param {String|zebkit.draw.TextRender|zebkit.data.TextModel} [txt] a text string,
     * text model or text render instance
     * @param {Integer} [w] an image preferred width
     * @param {Integer} [h] an image preferred height
     * @constructor
     * @class zebkit.ui.ImageLabel
     * @extends zebkit.ui.Panel
     */
    pkg.ImageLabel = Class(pkg.Panel, [
        function(txt, path, w, h) {
            var img = null,
                lab = null;

            if (arguments.length > 0) {
                lab = zebkit.instanceOf(txt, pkg.Panel) ? txt
                                                        : new this.clazz.Label(txt);
                if (arguments.length > 1) {
                    img = zebkit.instanceOf(path, pkg.ImagePan) ? path
                                                                : new this.clazz.ImagePan(path);
                    if (arguments.length > 2) {
                        img.setPreferredSize(w, (arguments.length > 3 ? h : w));
                    }
                }
            }

            // TODO: this is copy paste of Panel constructor to initialize fields that has to
            // be used for adding child components. these components have to be added before
            // properties() call. a bit dirty trick
            if (this.kids === undefined) {
                this.kids = [];
            }

            this.layout = new zebkit.layout.FlowLayout("left", "center", "horizontal", 6);

            // add before panel constructor thanks to copy pasted code above
            if (img !== null) {
                this.add(img);
            }

            if (lab !== null) {
                this.add(lab);
            }

            this.$super();

            lab.setVisible(txt !== null);
        },

        function $clazz() {
            this.ImagePan = Class(pkg.ImagePan, []);
            this.Label    = Class(pkg.Label, []);
        },

        function $prototype() {
            /**
             * Set the specified caption
             * @param {String|zebkit.ui.Label} c a label text or component
             * @method setValue
             * @chainable
             */
            this.setValue = function(c) {
                var lab = this.getLabel();

                if (zebkit.instanceOf(c, pkg.Label)) {
                    var i = -1;
                    if (lab !== null) {
                        i = this.indexOf(lab);
                    }

                    if (i >= 0) {
                        this.setAt(i, c);
                    }
                } else {
                    lab.setValue(c);
                    lab.setVisible(c !== null);
                }

                return this;
            };

            /**
             * Set the specified label image
             * @param {String|Image} p a path to an image of image object
             * @method setImage
             * @chainable
             */
            this.setImage = function(p) {
                var image = this.getImagePan();
                image.setImage(p);
                image.setVisible(p !== null);
                return this;
            };

            /**
             * Get image panel.
             * @return  {zebkit.ui.ImagePan} an image panel.
             * @method getImagePan
             */
            this.getImagePan = function() {
                return this.byPath("/~zebkit.ui.ImagePan");
            };

            /**
             * Get label component.
             * @return  {zebkit.ui.ImagePan} a label component.
             * @method getLabel
             */
            this.getLabel = function(p) {
                return this.byPath("/~zebkit.ui.Label");
            };

            /**
             * Set the caption font
             * @param {zebkit.Font} a font
             * @method setFont
             * @chainable
             */
            this.setFont = function() {
                var lab = this.getLabel();
                if (lab !== null) {
                    lab.setFont.apply(lab, arguments);
                }
                return this;
            };

            /**
             * Set the caption color
             * @param {String} a color
             * @method setColor
             * @chainable
             */
            this.setColor = function (c) {
                var lab = this.getLabel();
                if (lab !== null) {
                    lab.setColor(c);
                }
                return this;
            };

            /**
             * Get caption value
             * @return {zebkit.ui.Panel} a caption value
             * @method getValue
             */
            this.getValue = function () {
                var lab = this.getLabel();
                return lab === null ? null : lab.getValue();
            };

            /**
             * Set the image alignment.
             * @param {String} an alignment. Following values are possible:
             *
             *    - "left"
             *    - "right"
             *    - "top"
             *    - "bottom"
             *
             * @method  setImgAlignment
             * @chainable
             */
            this.setImgAlignment = function(a) {
                var b   = false,
                    img = this.getImagePan(),
                    i   = this.indexOf(img);

                if (a === "top" || a === "bottom") {
                    if (this.layout.direction !== "vertical") {
                        this.layout.direction = "vertical";
                        b = true;
                    }
                } else if (a === "left" || a === "right") {
                    if (this.layout.direction !== "horizontal") {
                        this.layout.direction = "horizontal";
                        b = true;
                    }
                }

                if (this.layout.ax !== "center") {
                    this.layout.ax = "center";
                    b = true;
                }

                if (this.layout.ay !== "center") {
                    this.layout.ay = "center";
                    b = true;
                }

                if ((a === "top" || a === "left") && i !== 0 ) {
                    this.insert(null, 0, this.removeAt(i));
                    b = false;
                } else if ((a === "bottom"  || a === "right") && i !== 1) {
                    this.add(null, this.removeAt(i));
                    b = false;
                }

                if (b) {
                    this.vrp();
                }

                return this;
            };

            /**
             * Set image preferred size.
             * @param {Integer} w a width and height if the second argument has not been specified
             * @param {Integer} [h] a height
             * @method setImgPreferredSize
             * @chainable
             */
            this.setImgPreferredSize = function (w, h) {
                if (arguments.length === 1) {
                    h = w;
                }
                this.getImagePan().setPreferredSize(w, h);
                return this;
            };
        }
    ]);

    /**
     * Progress bar UI component class.
     * @class zebkit.ui.Progress
     * @constructor
     * @param {String} [orient] an orientation of the progress bar. Use
     * "vertical" or "horizontal" as the parameter value
     * @extends zebkit.ui.Panel
     */

    /**
     * Fired when a progress bar value has been updated
     *
     *     progress.on(function(src, oldValue) {
     *         ...
     *     });
     *
     *  @event fired
     *  @param {zebkit.ui.Progress} src a progress bar that triggers
     *  the event
     *  @param {Integer} oldValue a progress bar previous value
     */
    pkg.Progress = Class(pkg.Panel, [
        function(orient) {
            this.$super();
            if (arguments.length > 0) {
                this.setOrientation(orient);
            }
        },

        function $prototype() {
            /**
             * Progress bar value
             * @attribute value
             * @type {Integer}
             * @readOnly
             */
            this.value = 0;

            /**
             * Progress bar element width
             * @attribute barWidth
             * @type {Integer}
             * @readOnly
             * @default 6
             */
             this.barWidth = 6;

            /**
             * Progress bar element height
             * @attribute barHeight
             * @type {Integer}
             * @readOnly
             * @default 6
             */
            this.barHeight = 6;

            /**
             * Gap between bar elements
             * @default 2
             * @attribute gap
             * @type {Integer}
             * @readOnly
             */
            this.gap = 2;

            /**
             * Progress bar maximal value
             * @attribute maxValue
             * @type {Integer}
             * @readOnly
             * @default 20
             */
            this.maxValue = 20;

            /**
             * Bar element view
             * @attribute barView
             * @readOnly
             * @type {String|zebkit.draw.View}
             * @default "blue"
             */
            this.barView = "blue";


            /**
             * Title element view .
             * @attribute titleView
             * @readOnly
             * @type {zebkit.draw.View}
             * @default null
             */
            this.titleView = null;

            /**
             * Progress bar orientation
             * @default "horizontal"
             * @attribute orient
             * @type {String}
             * @readOnly
             */
            this.orient = "horizontal";

            this.paint = function(g){
                var left    = this.getLeft(),
                    right   = this.getRight(),
                    top     = this.getTop(),
                    bottom  = this.getBottom(),
                    rs      = (this.orient === "horizontal") ? this.width - left - right
                                                             : this.height - top - bottom,
                    barSize = (this.orient === "horizontal") ? this.barWidth
                                                             : this.barHeight;

                if (rs >= barSize){
                    var vLoc   = Math.floor((rs * this.value) / this.maxValue),
                        x      = left,
                        y      = this.height - bottom,
                        bar    = this.barView,
                        wh     = this.orient === "horizontal" ? this.height - top - bottom
                                                              : this.width - left - right;

                    while (x < (vLoc + left) && this.height - vLoc - bottom < y){
                        if (this.orient === "horizontal"){
                            bar.paint(g, x, top, barSize, wh, this);
                            x += (barSize + this.gap);
                        } else {
                            bar.paint(g, left, y - barSize, wh, barSize, this);
                            y -= (barSize + this.gap);
                        }
                    }

                    if (this.titleView !== null) {
                        var ps = this.barView.getPreferredSize();
                        this.titleView.paint(g, Math.floor((this.width  - ps.width ) / 2),
                                                Math.floor((this.height - ps.height) / 2),
                                                ps.width, ps.height, this);
                    }
                }
            };

            this.calcPreferredSize = function(l) {
                var barSize = (this.orient === "horizontal") ? this.barWidth
                                                             : this.barHeight,
                    v1 = (this.maxValue * barSize) + (this.maxValue - 1) * this.gap,
                    ps = this.barView.getPreferredSize();

                ps = (this.orient === "horizontal") ? {
                                                         width :v1,
                                                         height:(this.barHeight >= 0 ? this.barHeight
                                                                                        : ps.height)
                                                      }
                                                    : {
                                                        width:(this.barWidth >= 0 ? this.barWidth
                                                                                     : ps.width),
                                                        height: v1
                                                      };
                if (this.titleView !== null) {
                    var tp = this.titleView.getPreferredSize();
                    ps.width  = Math.max(ps.width, tp.width);
                    ps.height = Math.max(ps.height, tp.height);
                }
                return ps;
            };

            /**
             * Set the progress bar orientation
             * @param {String} o an orientation: "vertical" or "horizontal"
             * @method setOrientation
             * @chainable
             */
            this.setOrientation = function(o) {
                if (o !== this.orient) {
                    this.orient = zebkit.util.validateValue(o, "horizontal", "vertical");
                    this.vrp();
                }
                return this;
            };

            /**
             * Set maximal integer value the progress bar value can rich
             * @param {Integer} m a maximal value the progress bar value can rich
             * @method setMaxValue
             * @chainable
             */
            this.setMaxValue = function(m) {
                if (m !== this.maxValue) {
                    this.maxValue = m;
                    this.setValue(this.value);
                    this.vrp();
                }
                return this;
            };

            /**
             * Set the current progress bar value
             * @param {Integer} p a progress bar
             * @method setValue
             * @chainable
             */
            this.setValue = function(p) {
                p = p % (this.maxValue + 1);
                if (this.value !== p){
                    var old = this.value;
                    this.value = p;
                    this.fire("fired", [this, old]);
                    this.repaint();
                }
                return this;
            };

            /**
             * Set the given gap between progress bar element elements
             * @param {Integer} g a gap
             * @method setGap
             * @chainable
             */
            this.setGap = function(g) {
                if (this.gap !== g){
                    this.gap = g;
                    this.vrp();
                }
                return this;
            };

            /**
             * Set the progress bar element element view
             * @param {zebkit.draw.View} v a progress bar element view
             * @method setBarView
             * @chainable
             */
            this.setBarView = function(v) {
                if (this.barView != v){
                    this.barView = zebkit.draw.$view(v);
                    this.vrp();
                }
                return this;
            };

            /**
             * Set the progress bar element size
             * @param {Integer} w a element width
             * @param {Integer} h a element height
             * @method setBarSize
             * @chainable
             */
            this.setBarSize = function(w, h) {
                if (w !== this.barWidth && h !== this.barHeight){
                    this.barWidth  = w;
                    this.barHeight = h;
                    this.vrp();
                }
                return this;
            };
        }
    ]).events("fired");
});